<?php

namespace Drupal\snippet_manager\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Form builder for "Edit variable" form.
 *
 * @todo: Add plugin form validation.
 */
class VariableEditForm extends VariableFormBase {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    $form = parent::form($form, $form_state);

    $form['configuration'] = $this
      ->getPlugin()
      ->buildConfigurationForm($form, $form_state);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actionsElement(array $form, FormStateInterface $form_state) {
    $element = parent::actionsElement($form, $form_state);

    $route_parameters = [
      'snippet' => $this->entity->id(),
      'variable' => $this->getVariableName(),
    ];

    $element['delete']['#url'] = Url::fromRoute('snippet_manager.variable_delete_form', $route_parameters);

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function buildEntity(array $form, FormStateInterface $form_state) {

    /** @var \Drupal\snippet_manager\Entity\Snippet $entity */
    $entity = parent::buildEntity($form, $form_state);

    if ($form_state->isSubmitted()) {

      $this->plugin->submitConfigurationForm($form, $form_state);

      $variable_name = $this->getVariableName();
      $variable = $entity->getVariable($variable_name);
      $variable['configuration'] = $this->plugin->getConfiguration();
      $entity->setVariable($variable_name, $variable);
    }

    return $entity;
  }

}
